@@ -50,8 +50,6 @@ class _Feature(object):
         raise NotImplementedError
 
 
-
-
 class Point(_Feature):
     """
     A zero dimensional feature
@@ -296,6 +294,14 @@ class LinearRing(LineString):
         if self._geoms[0].coords != self._geoms[-1].coords:
             self._geoms.append(self._geoms[0])
 
+    def _set_orientation(self, clockwise=False):
+        """ sets the orientation of the coordinates in
+        clockwise or counterclockwise (default) order"""
+        area = signed_area(self.coords)
+        if (area >= 0) and clockwise:
+            self._geoms = self._geoms[::-1]
+        elif (area < 0) and not clockwise:
+            self._geoms = self._geoms[::-1]
 
 
 class Polygon(_Feature):
@@ -386,8 +392,8 @@ class Polygon(_Feature):
             if isinstance(shell[0][0], (list, tuple)):
                 # we passed shell and holes in the first parameter
                 self._exterior = LinearRing(shell[0])
-                if len(shell) == 2:
-                    for hole in shell[1]:
+                if len(shell) > 1:
+                    for hole in shell[1:]:
                         self._interiors.append(LinearRing(hole))
             else:
                 self._exterior = LinearRing(shell)
@@ -424,6 +430,18 @@ class Polygon(_Feature):
                             ) + ')'
         return self._type.upper() + '(' + ec + ic + ')'
 
+
+    def _set_orientation(self, clockwise=False, exterior=True, interiors=True):
+        """ sets the orientation of the coordinates in
+        clockwise or counterclockwise (default) order"""
+        if exterior:
+            self.exterior._set_orientation(clockwise)
+        if interiors:
+            for interior in self.interiors:
+                interior._set_orientation(clockwise)
+
+
+
 class MultiPoint(_Feature):
     """A collection of one or more points
 
@@ -533,6 +551,13 @@ class MultiPoint(_Feature):
         wc = [ ' '.join([str(x) for x in c.coords[0]]) for c in self.geoms]
         return self._type.upper() + '(' + ', '.join(wc) + ')'
 
+    def __len__(self):
+        if self._geoms:
+            return len(self._geoms)
+        else:
+            return 0
+
+
 
 class MultiLineString(_Feature):
     """
@@ -618,6 +643,11 @@ class MultiLineString(_Feature):
                             ) + ')'
         return self._type.upper() + '(' + wc + ')'
 
+    def __len__(self):
+        if self._geoms:
+            return len(self._geoms)
+        else:
+            return 0
 
 class MultiPolygon(_Feature):
     """A collection of one or more polygons
@@ -729,6 +759,18 @@ class MultiPolygon(_Feature):
             pc += '(' + ec + ic + ')'
         return self._type.upper() + '(' + pc + ')'
 
+    def _set_orientation(self, clockwise=False, exterior=True, interiors=True):
+        """ sets the orientation of the coordinates in
+        clockwise or counterclockwise (default) order for all
+        contained polygons"""
+        for geom in self.geoms:
+            geom._set_orientation(clockwise, exterior, interiors)
+
+    def __len__(self):
+        if self._geoms:
+            return len(self._geoms)
+        else:
+            return 0
 
 class GeometryCollection(_Feature):
     """A heterogenous collection of geometries
@@ -744,19 +786,95 @@ class GeometryCollection(_Feature):
     So it's very rarely used in the real GIS professional world.
     """
     _type = 'GeometryCollection'
-#    @property
-#    def __geo_interface__(self):
-#        raise NotImplementedError
+    _geoms = None
 
-#    def __init__(self):
-#        raise NotImplementedError
+    @property
+    def __geo_interface__(self):
+        gifs = []
+        for geom in self._geoms:
+            gifs.append(geom.__geo_interface__)
+        return {'type': self._type, 'geometries': gifs }
 
-#    @property
-#    def geoms(self):
-#        raise NotImplementedError
+    def __init__(self, features):
+        self._geoms = []
+        if isinstance(features, (list, tuple)):
+            for feature in features:
+                if isinstance(feature, (Point, LineString, LinearRing, Polygon)):
+                    self._geoms.append(feature)
+                elif isinstance(as_shape(feature), (Point, LineString, LinearRing, Polygon)):
+                    self._geoms.append(as_shape(feature))
+                else:
+                    raise ValueError
+        else:
+            raise TypeError
+
+    @property
+    def geoms(self):
+        for geom in self._geoms:
+            if isinstance(geom, (Point, LineString, LinearRing, Polygon)):
+                yield geom
+            else:
+                raise ValueError("Illegal geometry type.")
+
+
+    @property
+    def bounds(self):
+        if self._geoms:
+            minx = self._geoms[0].bounds[0]
+            miny = self._geoms[0].bounds[1]
+            maxx = self._geoms[0].bounds[2]
+            maxy = self._geoms[0].bounds[3]
+            for geom in self.geoms:
+                minx = min(geom.bounds[0], minx)
+                miny = min(geom.bounds[1], miny)
+                maxx = max(geom.bounds[2], maxx)
+                maxy = max(geom.bounds[3], maxy)
+            return (minx, miny, maxx, maxy)
+
+    def to_wkt(self):
+        wkts = []
+        for geom in self.geoms:
+            wkts.append(geom.to_wkt())
+        return 'GEOMETRYCOLLECTION (%s)' % ', '.join(wkts)
+
+    def __len__(self):
+        if self._geoms:
+            return len(self._geoms)
+        else:
+            return 0
+
+
+def signed_area(coords):
+    """Return the signed area enclosed by a ring using the linear time
+    algorithm at http://www.cgafaq.info/wiki/Polygon_Area. A value >= 0
+    indicates a counter-clockwise oriented ring.
+    """
+    if len(coords[0]) == 2:
+        xs, ys = map(list, zip(*coords))
+    elif len(coords[0]) == 3:
+        xs, ys, zs = map(list, zip(*coords))
+    else:
+        raise ValueError
+    xs.append(xs[1])
+    ys.append(ys[1])
+    return sum(xs[i]*(ys[i+1]-ys[i-1]) for i in range(1, len(coords)))/2.0
+
+
+def orient(polygon, sign=1.0):
+    s = float(sign)
+    rings = []
+    ring = polygon.exterior
+    if signed_area(ring.coords)/s >= 0.0:
+        rings.append(ring.coords)
+    else:
+        rings.append(list(ring.coords)[::-1])
+    for ring in polygon.interiors:
+        if signed_area(ring.coords)/s <= 0.0:
+            rings.append(ring.coords)
+        else:
+            rings.append(list(ring.coords)[::-1])
+    return Polygon(rings[0], rings[1:])
 
-#    def to_wkt(self):
-#        raise NotImplementedError
 
 def as_shape(feature):
     """ creates a pygeoif feature from an object that
@@ -766,6 +884,8 @@ def as_shape(feature):
     if isinstance(feature, dict):
         if ('coordinates' in feature) and ('type' in feature):
             gi = feature
+        elif (feature['type'] == 'GeometryCollection') and ('geometries' in feature):
+            gi = feature
     elif hasattr(feature, '__geo_interface__'):
         gi = feature.__geo_interface__
     else:
@@ -774,11 +894,18 @@ def as_shape(feature):
             cdict = dict(feature)
             if ('coordinates' in cdict) and ('type' in cdict):
                 gi = cdict
+            elif (cdict['type'] == 'GeometryCollection') and ('geometries' in cdict):
+                gi = cdict
         except:
             pass
     if gi:
-        coords = gi['coordinates']
         ft = gi['type']
+        if ft == 'GeometryCollection':
+            features = []
+            for fi in gi['geometries']:
+                features.append(as_shape(fi))
+            return GeometryCollection(features)
+        coords = gi['coordinates']
         if ft == 'Point':
             return Point(coords)
         elif ft == 'LineString':
@@ -808,6 +935,7 @@ wkt_regex = re.compile(r'^(SRID=(?P<srid>\d+);)?'
     r'[ACEGIMLONPSRUTYZ\d,\.\-\(\) ]+)$',
     re.I)
 
+gcre = re.compile(r'POINT|LINESTRING|LINEARRING|POLYGON')
 
 outer = re.compile("\((.+)\)")
 inner = re.compile("\([^)]*\)")
@@ -877,6 +1005,15 @@ def from_wkt(geo_str):
                 exteriors = None
             polygons.append(Polygon([c.split() for c in coords[0]], exteriors))
         return MultiPolygon(polygons)
+    elif ftype == 'GEOMETRYCOLLECTION':
+        gc_type = gcre.findall(coordinates)
+        gc_coords = gcre.split(coordinates)[1:]
+        assert(len(gc_type)==len(gc_coords))
+        features = []
+        for i in range(0,len(gc_type)):
+            gc_wkt = gc_type[i] + gc_coords[i][:gc_coords[i].rfind(')') +1]
+            features.append(from_wkt(gc_wkt))
+        return GeometryCollection(features)
     else:
         raise NotImplementedError
 
